
import com.cattura.agents.hardware.web.vos.RequestResponseHelper
import org.webbitserver.HttpControl
import com.cattura.agents.configuration_module.model.proxies.EnvironmentProxy
import org.webbitserver.HttpHandler
import org.webbitserver.HttpRequest
import org.webbitserver.HttpResponse

class MediaInfoHttpHandler implements HttpHandler
{
    private static final List ALLOWED_ROLES = ["Content manager"]
    private static final String PERMISSION = "analyze media"    
    
    private final def _environment
    private final def _analyzers
    
    public MediaInfoHttpHandler (final EnvironmentProxy $environment, final Map $analyzers)
    {
        _environment = $environment
        _analyzers = $analyzers
    }
    
    public void handleHttpRequest (final HttpRequest $request, final HttpResponse $response, final HttpControl $control)
    {
        if ($request.uri().startsWith("/analyze/"))
        {
            def relativeRoot = new RequestResponseHelper($request, $response).getRelativeRoot()
            def mediaPath = $request.uri().substring("/analyze/".length())
            def mediaFile = new File(_environment.getStorageDirPath().getAbsolutePath() + File.separator + mediaPath);

            if (!mediaFile.exists())
            {
                $response.header("Content-Type","text/html").status(404).content("404 Not Found: ${mediaPath}").end()
                return
            }

            if (!mediaFile.isFile())
            {
                $response.header("Content-Type","text/html").status(400).content("400 Bad Request: ${mediaPath}").end()
                return                        
            }

            $response.header("Content-Type", "text/html").content("""\
            <html>
                <head>
                    <title>Media Analyzer - ${mediaPath}</title>

                    <link rel="stylesheet" type="text/css" href="${relativeRoot}jquery-ui/css/bootstrap.css" />
                    <script type="text/javascript" src="${relativeRoot}jquery-ui/js/jquery.js"></script>
                    <script type="text/javascript" src="${relativeRoot}jquery-ui/js/bootstrap.js"></script>

                    <style type="text/css">
                    body {
                        margin: 5px;
                    }
                    #upper-container {
                        width: 100%;
                        height: 100%;
                        margin-bottom: 10px;
                    }
                    #upper-container h1, span {
                        margin: 0px;
                        padding: 5px;
                    }
                    #upper-container span {
                        font-weight: bold;
                    }
                    #tabs {
                        margin-top: 5px;
                    }
                    .ui-tabs-panel {
                        padding: 5px;
                        height: 100%;
                    }
                    iframe {
                        width: 100%;
                        height: 100%;
                        border: 0px;
                    }
                    </style>
                    <script type="text/javascript">
                    jQuery(function(q) {
                        var mediaPath = "${mediaPath}";
                        var hash = window.location.hash || '#mediainfo';

                        q("#tabs li a").click(function(event) {
                            var hash = q(this).attr("href");
                            var tabs = q("#tabs");
                            tabs.find("iframe").attr("src","about:blank");
                            tabs.find(hash).find("iframe").attr("src",'${relativeRoot}' + hash.substr(1) + '/' + mediaPath);
                        });

                        q("#tabs li a[href='"+hash+"']").click();
                    });
                    </script>
                </head>
                <body>
                    <div id="upper-container" class="ui-widget ui-widget-content ui-corner-all">
                        <h1>Cattura CaptureCast Media Analyzer</h1>
                        <span>Analyzing: ${mediaPath}</span>
                        <div id="tabs">
                            <ul class="nav nav-tabs" role="tablist">
                                <li class="active"><a href="#mediainfo" role="tab" data-toggle="tab">MediaInfo</a></li>
                                <li><a href="#atomicparsley" role="tab" data-toggle="tab">AtomicParsley</a></li>
                                <li><a href="#ffprobe" role="tab" data-toggle="tab">ffprobe</a></li>
                            </ul>

                            <div class="tab-content">
                                <div class="tab-pane active" id="mediainfo" class="tab-pane active"><iframe src="about:blank"></iframe></div>
                                <div class="tab-pane" id="atomicparsley"><iframe src="about:blank"></iframe></div>
                                <div class="tab-pane" id="ffprobe"><iframe src="about:blank"></iframe></div>
                            </div>
                        </div>
                    </div>
                </body>
            </html>
            """).end()

            return
        }
        else
        {
            for (entry in _analyzers)
            {
                if ($request.uri().startsWith("/${entry.key}/"))
                {
                    def mediaPath = $request.uri().substring("/${entry.key}/".length())
                    def mediaFile = new File(_environment.getStorageDirPath().getAbsolutePath() + File.separator + mediaPath);

                    if (!mediaFile.exists())    
                    {
                        $response.header("Content-Type","text/html").status(404).content("404 Not Found: ${mediaPath}").end()
                        return
                    }

                    if (!mediaFile.isFile())
                    {
                        $response.header("Content-Type","text/html").status(400).content("400 Bad Request: ${mediaPath}").end()
                        return                        
                    }

                    try
                    {
                        def mediaInfo = entry.value.replace("{mediaFile}",mediaFile.absolutePath).execute()
                        mediaInfo.waitFor()

                        def responseText = mediaInfo.in.text

                        if (responseText.empty)
                        {
                            responseText = mediaInfo.err.text
                        }

                        $response.header("Content-Type",responseText.startsWith("<html>") ? "text/html" : "text/plain").content(responseText).end()
                    }
                    catch (final IOException $exception)
                    {
                        $response.header("Content-Type","text/html").status(500).content("500 Error: ${$exception.message}").end()                        
                    }

                    return;
                }
            }                    
        }                

        $control.nextHandler()
    }
}